// bd_manager.js - TCC_Crypto's module
// author: vitor_lp

exports.criarBancoDeDadosLocal = criarBancoDeDadosLocal;
exports.inserirChavesPublicas = inserirChavesPublicas;
exports.inserirChavePrivada = inserirChavePrivada;
exports.inserirUsuario = inserirUsuario;
exports.inserirAmizade = inserirAmizade;
exports.selecionaUsuarioPorNome = selecionaUsuarioPorNome;
exports.selecionaUsuarioPorNomeRemoto = selecionaUsuarioPorNomeRemoto;

var lm = require ("./math_lib");
var cm = require ("controller/comunicador");
var Request = require("sdk/request").Request;
var sqlite = require("./sqlite");
// conecta no banco local
sqlite.connect("tcc_crypto.sqlite");



/***********************************************************
 *  Cria o banco de dados local (um banco sqlite).
 *  Este banco guarda a chave privada do usuário
 * e as informações referentes ao seu grupo de contatos
 * (estas informações também estão no servidor, mas 
 * são guardadas localmente para evitar overheads pro conta
 * da comunicação com o servidor)
 ***********************************************************/
function criarBancoDeDadosLocal (){

    sqlite.execute("create table if not exists Usuario(id_usuario integer primary key, " +
                    "nome text, senha text, email text, chave_privada text, coprimo text);");
    
    /* Campos:
            id_amizade: é o identificador do campo
            id_dono_grupo: é o id do usuário dono do grupo (as pessoas são amigas dele)
            id_amigo: id do usuário que é amigo do dono do grupo
            coprimo: um valor que deve ser coprimo com os valores dos amigos
            chave_publica_g: a chave pública g do usuário id_amigo
            chave_publica_a: chave pública a do usuário id_amigo
    */
    sqlite.execute("create table if not exists Amizade(id_amizade integer primary key," +
                     "id_dono_grupo integer, id_amigo integer, coprimo text," +
                     "chave_publica_g text, chave_publica_a text);");
    
}

function inserirAmizade (nomeDono, nomeAmigo){
    
    lm = require ("./math_lib");
    N = lm.geraPrimo ();
    cm.enviaMensagem ("Gerou coprimo: " + N);
    var envia = Request({
      url: "http://54.213.65.77/insereAmizade.php?nomeDono="+nomeDono+"&nomeAmigo="+nomeAmigo+"&coprimo="+N,
      overrideMimeType: "text/plain; charset=latin1",
      onComplete: function (amizade) {
       
        // amizade é um objeto com os campos idAmizade, nomeAmigo, idAmigo, g, a, idDono
        // ou null (caso a inserção não tenha dado certo)
        try{
            //cm.enviaMensagem ("devolvido pelo servidor:\n amizade:" + amizade + "\n amizade.text: " + amizade.text);
           // cm.enviaMensagem ("---" + amizade.text + "---");
            amizade = JSON.parse (amizade.text);
            if (amizade == null){
                cm.notificar ("Não criou amizade", "provavelmente o usuário " + nomeAmigo + " não existe.");
            }else {
                console.log ("vai iserir:\nidAmizade"+amizade.idAmizade+"\nidDono: " + amizade.idDono + "\nidAmigo:" + amizade.idAmigo +
                            "\nN" + N + "\ng:" + amizade.g + "\na:" + amizade.a);
                sqlite.execute ("INSERT INTO Amizade(id_amizade, id_dono_grupo, id_amigo, coprimo,"
                            +"chave_publica_g, chave_publica_a) VALUES("+amizade.idAmizade
                            +","+amizade.idDono+","+amizade.idAmigo+",'"+N+"','"+amizade.g+"','"+
                            amizade.a+"')");
            }
        }catch (SyntaxError){
          cm.enviaMensagem ("ERRO DE SINTAXE\n ao fazer o parsing da amizade.");  
        }
      }
    });
    envia.get();
    //}
}



/*************************************************
*   Insere no banco de dados remoto as chaves
* públicas g e a.
***************************************************/
function inserirChavesPublicas (nome, senha, g, a){
    var envia = Request({
      url: "http://54.213.65.77/insere.php?nome="+nome+"&s="+senha+"&g="
                + g + "&a=" + a,
      overrideMimeType: "text/plain; charset=latin1",
      onComplete: function (response) {
                    cm.enviaMensagem ("Inseriu chaves públicas\n response: " + response );
                    return response;
                  }
    });
    envia.get();
}


/***************************************************************
 *      Insere a chave privada x na máquina cliente.
 *      Assume que o usuário cujo nome foi passado já existe no BD local.
 *      Usa o banco de dados SQLite.
 ****************************************************************/
function inserirChavePrivada (nome, x){

    // conecta no banco local
   // sqlite.connect("tcc_crypto.sqlite");
    // insere no banco local
    sqlite.execute('update Usuario set chave_privada="'+x+'" where nome="'+nome+'"');
    // fecha a conexão (está dando problema quando abro e fecho a conexão mais de uma vez)
    //sqlite.close();
    return "OK";
}

/*********************************************************************
 *      Insere um novo usuário no banco de dados local e no servidor.
 **********************************************************************/
function inserirUsuario (nome, email, senha, g, a, x){

    var b64 = require("sdk/base64");

    // insere no banco remoto
    inserirUsuarioRemoto (nome, b64.encode(email), senha,
        // função para processar o id do usuário inserido
        function (id){
            
            // insere no banco local
            sqlite.execute('insert into Usuario(id_usuario, nome, email, senha)'+
                            'values(' + parseInt (id, 10) + ',"'+nome+'", "'+email+'", "'+senha+'");');
                            
            inserirChavesPublicas (nome, senha, g, a);
            inserirChavePrivada (nome, x);
            
        });
}


function inserirUsuarioRemoto (nome, email, senha, callBack){
    var envia = Request({
      url: "http://54.213.65.77/insereUsuario.php?nome="+nome+"&email="+email+"&senha="+senha,
      overrideMimeType: "text/plain; charset=latin1",
      onComplete: function (id) {
                    callBack (id.text);
                  }
    });
    envia.get();
}

/***************************************************************
 *      Faz um select no banco de dados remoto procurando pelo
 * usuário cujo nome seja igual ao nome passado e executa
 * a função callBack passando null, caso nenhum usuário seja encontrado,
 * ou passando um objeto contendo os seguintes campos:
 * 
 * id    --> id do usuário buscado
 * nome  --> nome do usuário
 * email --> o e-mail cadastrado pelo usuário
 * senha --> senha guardada no banco de dados (md5)
 * g     --> chave pública g do usuário buscado
 * a     --> chave pública a do usuário buscado
 * chaveAmigoG  --> um array com as chaves públicas 'g' de cada contato
 * chaveAmigoA  --> um array com as chaves públicas 'a' de cada contato
 * coprimo      --> um array com os coprimos referentes a cada contato
 * idAmigo      --> um array com os ids de cada contato
 * 
 ****************************************************************/                
function selecionaUsuarioPorNomeRemoto (nome, callBack){
    var user = 1;
    var envia = Request({
      url: "http://54.213.65.77/devolve.php?sql=selecionaUsuarioPorNome&nome="
            + nome,
      overrideMimeType: "text/plain; charset=latin1",
      onComplete: function (response) {
                    try{
                        user =  JSON.parse (response.text);
                    }catch (SyntaxError){
                        cm.enviaMensagem ("ERRO DE SINTAXE");
                        user = null;
                    }
                    callBack (user);
                  }
    });
    
    envia.get();
    return user;
}

  
function selecionaUsuarioPorNome (nome, callBack){
    
    sqlite.execute("SELECT Usuario.id_usuario, Usuario.senha, Usuario.email, " +
                    "Usuario.chave_privada FROM Usuario " + 
                    "WHERE Usuario.nome = '"+nome+"'",
        function (resp, stt){
            /*
                XXX:
                TODO:
                     Se não encontrar o usuário no banco de dados local,
                    é necessário procurar no banco de dados remoto.
                     Se ele existir no remoto, gerar novas chaves e inserir
                    tudo no banco local (nome, senha, id, chaves, amizades, etc...)
            */
            if (resp.rows == 0){ // se não encontrou o usuário cujo nome foi passado
            
                
            
                /* tenta achar o usuário no banco de dados remoto */
                selecionaUsuarioPorNomeRemoto (nome, function (usrTmp){
                /** 
                 FALTA  chavePrivada e qntAmigos
                 SOBRANDO
                 * g     --> chave pública g do usuário buscado
                 * a     --> chave pública a do usuário buscado
                 */                   
                 
                 console.log ("** -Não encontrou " + nome);
                    // gerar as chaves
                    [g, a, x] = lm.geraChaves();
                    console.log ("0 -Não encontrou " + usrTmp);
                    // insere no banco local
                    sqlite.execute('insert into Usuario(id_usuario, nome, email, senha)'+
                                    'values(' + parseInt (usrTmp.id, 10) + ',"'+nome+'", "'+usrTmp.email+'", "'+usrTmp.senha+'");');
                    inserirChavesPublicas (nome, usrTmp.senha, usrTmp.g, usrTmp.a);
                    console.log ("1 - Não encontrou " + nome);   
                    inserirChavePrivada (nome, x);
                    console.log ("2 - Não encontrou " + nome);   
                    callBack (usrTmp);  // chama a função de callBack com null
                });
                
            }else {
                var id = resp.data[0][0];
                var senha = resp.data[0][1];
                var email = resp.data[0][2];
                var chavePrivada = resp.data[0][3];
                // resp vai ser um objeto no padrão JSON
                var resp = '{"id":'+ id+', "nome": "'+nome+'","senha":"'+senha+
                                '", "email":"'+email+'", "chavePrivada":"'+chavePrivada + '"';
                // seleciona as amizades do usuário
                sqlite.execute("SELECT Amizade.id_dono_grupo, Amizade.id_amigo," +
                            "Amizade.coprimo, Amizade.chave_publica_g, Amizade.chave_publica_a " +
                            "FROM Amizade WHERE Amizade.id_dono_grupo = " + id + "",
                    function(result,status){
                        // atributo dizendo quantos amigos este usuário tem
                        resp = resp + ',"qntAmigos":'+result.rows;
                        idAmigo = '"idAmigo":[';
                        coprimo = '"coprimo":[';
                        chaveAmigoG = '"chaveAmigoG":[';
                        chaveAmigoA = '"chaveAmigoA":[';
                        for(var i=0;i<result.rows;i++){
                            idAmigo = idAmigo + ','+result.data[i][1];
                            coprimo = coprimo + ',"'+result.data[i][2] + '"';
                            chaveAmigoG = chaveAmigoG + ',"'+result.data[i][3] + '"';
                            chaveAmigoA = chaveAmigoA + ',"'+result.data[i][4] + '"';
                        }
                        idAmigo = idAmigo + ']';
                        coprimo = coprimo + ']';
                        chaveAmigoG = chaveAmigoG + ']';
                        chaveAmigoA = chaveAmigoA + ']';
                        resp = resp+','+idAmigo+','+coprimo+','+chaveAmigoG+','+chaveAmigoA+'}';
                        resp = resp.replace(/\[,/g, '[');
                        
                        usr = JSON.parse (resp);//converte: resposta  --> objeto
                        callBack (usr); // executa callBack passando o objeto criado
                    } // end anonimous function
                );
            } // end else
        } // end anonimous function
    ); // end sqlite.execute
} // end selecionaUsuarioPorNome